//**************************************************************************************************
//*  FRIDA: fast reliable interactive data analysis
//*  (C) Joachim Wuttke 2001-
//*  http://apps.jcns.fz-juelich.de/frida
//**************************************************************************************************

//! \file  fassign.cpp
//! \brief Collection of assignment functions.

#include "defs.hpp"


#include "fregistry.hpp"
#include "func.hpp"
#include "obj.hpp"
#include "ptr.hpp"
#include "variables.hpp"


template <class A> A func_assign(string key, A v)
{
    SVariRegistry::instance()->register_scalar(key, CObj::objectify(v));
    return v;
}
void func_assign(double& r, double& dr, string key, double v, double dv)
{
    SVariRegistry::instance()->register_scalar(key, CObj::objectify(v, dv));
    r = v;
    dr = dv;
}

int func_defined(string key) { return SVariRegistry::instance()->find(key) ? 1 : 0; }

void func_undef(string key)
{
    bool ret = SVariRegistry::instance()->undef(key);
    if (!ret)
        throw "key " + key + " was not defined";
}

void fassign_initialize()
{
    auto G = SFuncRegistry::instance();
    CFuncMetadata m;

    m = { "=", 2, "a=b     [assignment]", 14 };
    G->register_fct_i_si(m, func_assign);
    G->register_fct_d_sd(m, func_assign);
    G->register_fct_e_se(m, func_assign);
    G->register_fct_s_ss(m, func_assign);

    m = { "defined", 1, "(s): 1 if there is a variable s" };
    G->register_fct_i_s(m, func_defined);
    m = { "undef", 1, "(s): undefine variable s, return 1 upon success" };
    G->register_fct_0_s(m, func_undef);
}
