//**************************************************************************************************
//*  FRIDA: fast reliable interactive data analysis
//*  (C) Joachim Wuttke 1990-, v2(C++) 2001-
//*  http://apps.jcns.fz-juelich.de/frida
//**************************************************************************************************

//! \file  genus.cpp
//! \brief Variable reference CGenus.

#include "defs.hpp"

#include "genus.hpp"


//**************************************************************************************************
//*  class CGenus
//**************************************************************************************************


//! Returns number contained in parameter label s, starting at index pos.

int CGenus::request_num(string s, int pos)
{
    if (s.size() <= pos)
        throw "missing parameter number in '" + s + "'";
    int ret;
    if (!triv::any2int(s.substr(pos), &ret))
        throw "invalid parameter number in '" + s + "'";
    return ret;
}


//! Creates instance according to given parameter label.

CGenus::CGenus(string s)
{
    num = 0;
    if (s.size() < 1)
        throw S("Variable name missing");

    if (s[0] == 'p') { // curve parameter
        num = request_num(s, 1);
        categ = VCateg::CP;
    } else if (s == "z+") { // new z value
        num = -1;
        categ = VCateg::Z;
    } else if (s[0] == 'z') { // existing z value
        num = request_num(s, 1);
        categ = VCateg::Z;
    } else if (s == "r+") { // new real par
        num = -1;
        categ = VCateg::R;
    } else if (s[0] == 'r') { // existing real par
        num = request_num(s, 1);
        categ = VCateg::R;
    } else if (s == "nk") {
        categ = VCateg::NK;
    } else if (s == "nj") {
        categ = VCateg::NJ;
    } else if (s == "ni") {
        categ = VCateg::NI;
    } else if (s == "x") {
        categ = VCateg::X;
    } else if (s == "y") {
        categ = VCateg::Y;
    } else if (s == "dy") {
        categ = VCateg::DY;
    } else if (s == "c") {
        categ = VCateg::C;
    } else if (s == "outcome") {
        categ = VCateg::COUTCOME;
    } else if (s == "chi2") {
        categ = VCateg::CCHI2;
    } else if (s == "nfev") {
        categ = VCateg::CNFEV;
    } else if (s == "R2") {
        categ = VCateg::CR2;
    } else {
        throw "Invalid variable '" + s + "'";
    }
}


//! Returns string representation of this variable.

string CGenus::genus_info() const
{
    string ret;
    switch (categ) {
    case VCateg::X:
        ret = "x";
        break;
    case VCateg::Y:
        ret = "y";
        break;
    case VCateg::DY:
        ret = "dy";
        break;
    case VCateg::C:
        ret = "c";
        break;
    case VCateg::Z:
        ret = "z";
        break;
    case VCateg::R:
        ret = "r";
        break;
    case VCateg::CP:
        ret = "p";
        break;
    case VCateg::COUTCOME:
        ret = "outcome_code";
        break;
    case VCateg::CNFEV:
        ret = "n_function_evaluations";
        break;
    case VCateg::CCHI2:
        ret = "chi2";
        break;
    case VCateg::CR2:
        ret = "R2";
        break;
    case VCateg::NK:
        ret = "nk";
        break;
    case VCateg::NJ:
        ret = "nj";
        break;
    case VCateg::NI:
        ret = "ni";
        break;
    default:
        throw S("BUG: no info available for unexpected variable category");
    }
    if (numbered()) {
        if (num == -1)
            ret += "+";
        else
            ret += S(num);
    }
    return ret;
}
