//**************************************************************************************************
//*  FRIDA: fast reliable interactive data analysis
//*  (C) Joachim Wuttke 1990-, v2(C++) 2001-
//*  http://apps.jcns.fz-juelich.de/frida
//**************************************************************************************************

//! \file  genus.hpp
//! \brief Variable reference CGenus.

#ifndef GENUS_H
#define GENUS_H


//! Variable category.

enum class VCateg : char {
    X,
    Y,
    Z,
    R, // data
    DY, // associated error
    C,
    CP, // curve, curve parameters
    COUTCOME,
    CCHI2,
    CNFEV,
    CR2, // fit outcome
    NK,
    NJ,
    NI, // number of files, of spectra in file, of points in spectrum
    _LAST_
};


//! Variable reference, composed of a category and a number, if applicable.

class CGenus
{
public:
    VCateg categ;
    int num;

    CGenus(VCateg categ, int num = 0) : categ(categ), num(num) {}
    CGenus(const string s);

    bool is_errvar() const { return categ == VCateg::DY; }
    bool fitoutcome() const
    {
        return categ == VCateg::COUTCOME || categ == VCateg::CCHI2
            || categ == VCateg::CR2 || categ == VCateg::CNFEV;
    }
    bool pointwise() const
    {
        return categ == VCateg::X || categ == VCateg::Y || categ == VCateg::DY;
    }
    bool curvewise() const { return categ == VCateg::C || categ == VCateg::CP || fitoutcome(); }
    bool slicewise() const { return curvewise() || categ == VCateg::Z || categ == VCateg::NI; }
    bool filewise() const { return categ == VCateg::R || categ == VCateg::NJ; }
    bool sessionwise() const { return categ == VCateg::NK; }
    bool numbered() const
    {
        return categ == VCateg::CP || categ == VCateg::R || categ == VCateg::Z;
    }

    string genus_info() const;

private:
    static int request_num(string s, int pos);
};

#endif // GENUS_H
