//**************************************************************************************************
//*  FRIDA: fast reliable interactive data analysis
//*  (C) Joachim Wuttke 1990-, v2(C++) 2001-
//*  http://apps.jcns.fz-juelich.de/frida
//**************************************************************************************************

//! \file  slice.hpp
//! \brief CSlice handles z dependent data, is base class for CSpec, CCurve.

#ifndef SLICE_H
#define SLICE_H


//! Virtual base class for CSpec and CCurve. Holds z dependent data.

class CSlice
{
public:
    vector<RObj> z;
    bool frozen;

    CSlice() : frozen(false) {}

    vector<string> z_str() const;
    virtual string info_line() const = 0;
};


//! A CSlice within a data file, holds a x-y-dy data set.

class CSpec : public CSlice
{
public:
    vector<double> x, y, dy;

    // modify:
    void push_xy(double _x, double _y);
    void push_xyd(double _x, double _y, double _dy);
    void clear();
    void remove(int begin, int end);
    void resize(int n, bool with_dy);

    // extract data:
    int size() const;
    void intpol(const vector<double>& in_x, vector<double>* out_y, vector<double>* out_dy) const;
    double norm(string info) const;
    bool has_step(double step) const;
    string info_line() const;
    inline bool has_dy() const { return dy.size() != 0; }
    bool dy_are_all_nonzero() const;
    PSpec binned(const vector<int>& breaks, const bool sampling_error) const;
};


//! A CSlice within a curve file, holds parameters and fit outcome.

class CCurve : public CSlice
{
public:
    vector<double> P;
    vector<char> ParAttr;
    int fitOutcome;
    int nFunctionEvaluations;
    double fitChi2;
    double fitR2;

    CCurve() : CSlice() { clear_fit_outcome(); }
    void clear_fit_outcome();
    void assert_attr() const;
    bool has_global_par() const;
    string info_line() const;
};

#endif // SLICE_H
