#include "gtest/gtest.h"
#include "../trivia/vector_ops.hpp"

// Tests where all categories are assigned a supercategory
TEST(VecOpsTest, HistoLimits) {
    std::vector<double> x { 1., 3., 13., 17. };
    std::vector<double> l = triv::histogram_limits( x );
    EXPECT_NEAR( 0., l[0], 1e-14 );
    EXPECT_NEAR( 2., l[1], 1e-14 );
    EXPECT_NEAR( 8., l[2], 1e-14 );
    EXPECT_NEAR(15., l[3], 1e-14 );
    EXPECT_NEAR(19., l[4], 1e-14 );
}

TEST(VecOpsTest, InsertSorted) {
    std::vector<double> A { 1.1, 1.3, 1.8 };
    std::vector<double> B { 2.1, 2.2, 2.4, 2.7 };
    auto f = [](const double a, const double b) -> bool { return (a-(long)(a))<=(b-(long(b))); };
    std::vector<double> C = triv::merge_sorted(A, B, f);
    EXPECT_EQ( C[0], A[0] );
    EXPECT_EQ( C[1], B[0] );
    EXPECT_EQ( C[2], B[1] );
    EXPECT_EQ( C[3], A[1] );
    EXPECT_EQ( C[4], B[2] );
    EXPECT_EQ( C[5], B[3] );
    EXPECT_EQ( C[6], A[2] );
}

TEST(VecOpsTest, InsertSortedInplace) {
    std::vector<double> A { 1.1, 1.3, 1.8 };
    std::vector<double> B { 2.1, 2.2, 2.4, 2.7 };
    auto f = [](const double a, const double b) -> bool { return (a-(long)(a))<=(b-(long(b))); };
    A = triv::merge_sorted(A, B, f);
    EXPECT_EQ( A[0], 1.1  );
    EXPECT_EQ( A[1], B[0] );
    EXPECT_EQ( A[2], B[1] );
    EXPECT_EQ( A[3], 1.3  );
    EXPECT_EQ( A[4], B[2] );
    EXPECT_EQ( A[5], B[3] );
    EXPECT_EQ( A[6], 1.8  );
}
